import pandas as pd
from scipy.stats import shapiro  # normality test
from sklearn.impute import SimpleImputer  # used for mean/median/mode imputing


def detect_outliers_iqr_columns(data: pd.DataFrame, columns: list) -> dict:
    """Detects and returns outliers for specified columns in a DataFrame.

    Args:
        data (pd.DataFrame): Pandas DataFrame.
        columns (list): List of column names to detect outliers.

    Returns:
        dict: A dictionary where each key is a column name and its value is a DataFrame of outliers and their indices for that column.
    """

    outliers_dict = {}

    for col in columns:
        if col in data.columns:
            Q1 = data[col].quantile(0.25)
            Q3 = data[col].quantile(0.75)
            IQR = Q3 - Q1
            lower_bound = Q1 - 1.5 * IQR
            upper_bound = Q3 + 1.5 * IQR

            outlier_condition = (data[col] < lower_bound) | (data[col] > upper_bound)
            outliers = data[col][outlier_condition]
            outliers_dict[col] = pd.DataFrame(
                {"Index": outliers.index, "Outlier": outliers.values}
            )

    return outliers_dict


def check_for_normality(
    df: pd.DataFrame, features: list[str], pvalue_threshold: float
) -> None:
    """Checks for normality in given features, useful in deciding how to impute.

    Args:
        df (pd.DataFrame): Pandas DataFrame
        features (list[str]): Features in DataFrame to check
        pvalue (float): P-Value threshold for normality check
    """

    for feature in features:
        p_value = shapiro(df[feature]).pvalue

        if p_value > pvalue_threshold:
            print(f"{feature} is normally distributed (p-value > {pvalue_threshold})")
        else:
            print(
                f"{feature} is not normally distributed (p-value <= {pvalue_threshold})"
            )


def impute_features(
    df: pd.DataFrame, features_to_impute: list[str], strat: str
) -> pd.DataFrame:
    """Imputes given features with the median value and returns a copy of the DataFrame.

    Args:
        df (pd.DataFrame): Pandas DataFrame
        features_to_impute (list[str]): Features to impute with the median of that feature

    Returns:
        pd.DataFrame: A new DataFrame with imputed values, leaving the original DataFrame unchanged.
    """

    df_copy = df.copy()

    imputer = SimpleImputer(strategy=strat)
    df_copy[features_to_impute] = imputer.fit_transform(df_copy[features_to_impute])

    return df_copy



from setuptools import setup, find_packages

setup(
    name="zoomds",
    version="0.57",
    author="Matthew Iversen",
    author_email="iversenmatt7@gmail.com",
    description="A collection of data science and analysis functions for data cleaning, analysis, modeling, and visualization.",
    packages=find_packages(),
    install_requires=[
        "colorlog",
        "matplotlib",
        "numpy",
        "pandas",
        "scipy",
        "scikit-learn",
        "pyyaml",
    ],
    python_requires=">=3.6",
)



import pandas as pd


def get_object_cols(df: pd.DataFrame) -> list:
    """
    Get a list of column names that have 'object' or categorical data type in a DataFrame.

    Args:
        df (pd.DataFrame): The DataFrame to analyze.

    Returns:
        list: A list of column names containing 'object' or categorical data type.
    """

    object_columns = []

    for col in df.columns:
        if df[col].dtype == "object" or isinstance(df[col].dtype, pd.CategoricalDtype):
            object_columns.append(col)

    return object_columns


def get_numerical_cols(df: pd.DataFrame) -> list:
    """
    Get a list of column names that have numerical data type in a DataFrame.

    Args:
        df (pd.DataFrame): The DataFrame to analyze.

    Returns:
        list: A list of column names containing numerical data type.
    """

    numerical_columns = []

    for col in df.columns:
        if pd.api.types.is_numeric_dtype(df[col]):
            numerical_columns.append(col)

    return numerical_columns


def get_object_bool_mask(df: pd.DataFrame) -> list[bool]:
    """
    Determine whether each column in the DataFrame is discrete or continuous.

    Args:
        dataframe (pd.DataFrame): The input DataFrame.

    Returns:
        list[bool]: A list of Boolean values, where True indicates a discrete feature,
            and False indicates a continuous feature.
    """

    discrete_features = [dtype == "object" for dtype in df.dtypes]

    return discrete_features
